@@ -28,29 +28,16 @@
 #include <glib/gi18n-lib.h>
 #include <string.h>
 
-#ifndef DOUBLE
+// We need multiple versions of this code.  We're going to include ourself
+// with different settings of various macros.  gdb will hate us.
+#include <goffice/goffice-multipass.h>
+#ifndef SKIP_THIS_PASS
 
-#define DOUBLE double
-#define SUFFIX(_n) _n
+#ifdef DEFINE_COMMON
 
-#define DISTRIBUTION_FIRST_PATH
-#ifdef GOFFICE_WITH_LONG_DOUBLE
-#include "go-distribution.c"
-#undef DISTRIBUTION_FIRST_PATH
-#undef DOUBLE
-#undef SUFFIX
-
-#ifdef HAVE_SUNMATH_H
-#include <sunmath.h>
-#endif
-#define DOUBLE long double
-#define SUFFIX(_n) _n ## l
-#endif
-#define DISTRIBUTION_LAST_PATH
-
-#endif
-
-#ifdef DISTRIBUTION_FIRST_PATH
+// The scale and location are always double.  Maybe reconsider that.
+#define DIST_SCALE ((DOUBLE)(dist->scale))
+#define DIST_LOCATION ((DOUBLE)(dist->location))
 
 enum {
 	DIST_PROP_0,
@@ -69,7 +56,7 @@ enum {
  * @GO_DISTRIBUTION_INVALID: (skip)
  **/
 
-static struct
+static const struct
 {
 	GODistributionType type;
 	char const *name;
@@ -81,12 +68,26 @@ static struct
 	{GO_DISTRIBUTION_LOGNORMAL, N_("Lognormal")}
 };
 
-char const *go_distribution_type_to_string (GODistributionType type)
+/**
+ * go_distribution_type_to_string:
+ * @type: distribution type to query
+ *
+ * Returns: an untranslated name of the distribution.
+ */
+char const *
+go_distribution_type_to_string (GODistributionType type)
 {
 	return DistributionTypes[type].name;
 }
 
-GODistributionType go_distribution_type_from_string (char const *name)
+/**
+ * go_distribution_type_from_string:
+ * @name: untranslated name of distribution
+ *
+ * Returns: type of distribution
+ */
+GODistributionType
+go_distribution_type_from_string (char const *name)
 {
 	int i = -1;
 	while (++i < GO_DISTRIBUTION_MAX)
@@ -108,17 +109,47 @@ typedef struct {
 	GODistributionType dist_type;
 
 	double (*get_density) (GODistribution *dist, double x);
-	double (*get_cumulative) (GODistribution *dist, double x);
+	double (*get_cumulative) (GODistribution *dist, gboolean lower_tail, double x);
 	double (*get_ppf) (GODistribution *dist, double x);
 
 #ifdef GOFFICE_WITH_LONG_DOUBLE
 	long double (*get_densityl) (GODistribution *dist, long double x);
-	long double (*get_cumulativel) (GODistribution *dist, long double x);
+	long double (*get_cumulativel) (GODistribution *dist, gboolean lower_tail, long double x);
 	long double (*get_ppfl) (GODistribution *dist, long double x);
 #endif
 
+#ifdef GOFFICE_WITH_DECIMAL64
+	_Decimal64 (*get_densityD) (GODistribution *dist, _Decimal64 x);
+	_Decimal64 (*get_cumulativeD) (GODistribution *dist, gboolean lower_tail, _Decimal64 x);
+	_Decimal64 (*get_ppfD) (GODistribution *dist, _Decimal64 x);
+#endif
 } GODistributionClass;
 
+#define SET_HANDLERS_D(K,PRE)				\
+	(K)->get_density = PRE ## _get_density;		\
+	(K)->get_cumulative = PRE ## _get_cumulative;	\
+	(K)->get_ppf = PRE ## _get_ppf;
+#ifdef GOFFICE_WITH_LONG_DOUBLE
+#define SET_HANDLERS_LD(K,PRE)				\
+	(K)->get_densityl = PRE ## _get_densityl;	\
+	(K)->get_cumulativel = PRE ## _get_cumulativel;	\
+	(K)->get_ppfl = PRE ## _get_ppfl;
+#else
+#define SET_HANDLERS_LD(K,PRE) (void)0
+#endif
+#ifdef GOFFICE_WITH_DECIMAL64
+#define SET_HANDLERS_D64(K,PRE)				\
+	(K)->get_densityD = PRE ## _get_densityD;	\
+	(K)->get_cumulativeD = PRE ## _get_cumulativeD;	\
+	(K)->get_ppfD = PRE ## _get_ppfD;
+#else
+#define SET_HANDLERS_D64(K,PRE) (void)0
+#endif
+
+
+#define SET_HANDLERS(K,PRE) do { SET_HANDLERS_D(K,PRE); SET_HANDLERS_LD(K,PRE); SET_HANDLERS_D64(K,PRE); } while (0)
+
+
 static void
 go_distribution_set_property (GObject *obj, guint param_id,
 			      GValue const *value, GParamSpec *pspec)
@@ -209,7 +240,7 @@ go_distribution_persist_sax_save (GOPersist const *gp, GsfXMLOut *output)
 			GValue	 value = { 0 };
 
 			g_value_init (&value, prop_type);
-			g_object_get_property  (G_OBJECT (dist), props[n]->name, &value);
+			g_object_get_property (G_OBJECT (dist), props[n]->name, &value);
 
 			/* No need to save default values */
 			if (g_param_value_defaults (props[n], &value)) {
@@ -225,6 +256,8 @@ go_distribution_persist_sax_save (GOPersist const *gp, GsfXMLOut *output)
 			case G_TYPE_UINT:
 			case G_TYPE_LONG:
 			case G_TYPE_ULONG:
+			case G_TYPE_INT64:
+			case G_TYPE_UINT64:
 			case G_TYPE_FLOAT:
 			case G_TYPE_DOUBLE:
 			case G_TYPE_ENUM:
@@ -258,13 +291,16 @@ go_distribution_persist_prep_sax (GOPersist *gp, GsfXMLIn *xin, xmlChar const **
 {
 	GODistribution *dist = GO_DISTRIBUTION (gp);
 	while (*attrs) {
-		if (!strcmp ((char const*) *attrs, "name") || !strcmp ((char const*) *attrs, "type"));
-		else if (!strcmp ((char const*) *attrs, "location"))
-			dist->location = g_strtod (attrs[1], NULL);
-		else if (!strcmp ((char const*) *attrs, "scale"))
-			dist->scale = g_strtod (attrs[1], NULL);
+		const char *attr = (const char *)(attrs[0]);
+		const char *val = (const char *)(attrs[1]);
+		if (!strcmp (attr, "name") || !strcmp (attr, "type"))
+			; // Nothing
+		else if (!strcmp (attr, "location"))
+			dist->location = g_ascii_strtod (val, NULL);
+		else if (!strcmp (attr, "scale"))
+			dist->scale = g_ascii_strtod (val, NULL);
 		else {
-			GParamSpec *spec = g_object_class_find_property (G_OBJECT_GET_CLASS (dist), *attrs);
+			GParamSpec *spec = g_object_class_find_property (G_OBJECT_GET_CLASS (dist), attr);
 			if (spec == NULL)
 				g_warning ("unknown property `%s' for class `%s'",
 					   *attrs, G_OBJECT_TYPE_NAME (gp));
@@ -274,8 +310,8 @@ go_distribution_persist_prep_sax (GOPersist *gp, GsfXMLIn *xin, xmlChar const **
 				case G_TYPE_DOUBLE: {
 					GValue value = {0};
 					g_value_init (&value, G_TYPE_DOUBLE);
-					g_value_set_double (&value, g_strtod (attrs[1], NULL));
-					g_object_set_property (G_OBJECT (dist), *attrs, &value);
+					g_value_set_double (&value, g_ascii_strtod (val, NULL));
+					g_object_set_property (G_OBJECT (dist), attr, &value);
 					g_value_unset (&value);
 					break;
 				}
@@ -308,109 +344,98 @@ go_distribution_scale (GODistribution *dist, double location, double scale)
 	dist->scale = scale;
 }
 
-#endif /* DISTRIBUTION_LAST_PATH */
+#endif /* DEFINE_COMMON */
 
 DOUBLE
-SUFFIX (go_distribution_get_density) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_distribution_get_density) (GODistribution *dist, DOUBLE x)
 {
 	GODistributionClass *go_dist_klass;
 
-	g_return_val_if_fail (GO_DISTRIBUTION (dist), SUFFIX (go_nan));
+	g_return_val_if_fail (GO_DISTRIBUTION (dist), SUFFIX(go_nan));
 
 	go_dist_klass = GO_DISTRIBUTION_GET_CLASS (dist);
-	if (go_dist_klass->SUFFIX (get_density) != NULL)
-		return go_dist_klass->SUFFIX (get_density) (dist, x);
-	return SUFFIX (go_nan);
+	if (go_dist_klass->SUFFIX(get_density) != NULL)
+		return go_dist_klass->SUFFIX(get_density) (dist, x);
+
+	return SUFFIX(go_nan);
 }
 
 DOUBLE
-SUFFIX (go_distribution_get_cumulative) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_distribution_get_cumulative) (GODistribution *dist, DOUBLE x)
 {
 	GODistributionClass *go_dist_klass;
 
-	g_return_val_if_fail (GO_DISTRIBUTION (dist), SUFFIX (go_nan));
+	g_return_val_if_fail (GO_DISTRIBUTION (dist), SUFFIX(go_nan));
 
 	go_dist_klass = GO_DISTRIBUTION_GET_CLASS (dist);
-	if (go_dist_klass->SUFFIX (get_cumulative) != NULL)
-		return go_dist_klass->SUFFIX (get_cumulative) (dist, x);
-	return SUFFIX (go_nan);
+	if (go_dist_klass->SUFFIX(get_cumulative) != NULL)
+		return go_dist_klass->SUFFIX(get_cumulative) (dist, TRUE, x);
+
+	return SUFFIX(go_nan);
 }
 
 DOUBLE
-SUFFIX (go_distribution_get_ppf) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_distribution_get_ppf) (GODistribution *dist, DOUBLE x)
 {
 	GODistributionClass *go_dist_klass;
 
-	g_return_val_if_fail (GO_DISTRIBUTION (dist), SUFFIX (go_nan));
+	g_return_val_if_fail (GO_DISTRIBUTION (dist), SUFFIX(go_nan));
 
 	go_dist_klass = GO_DISTRIBUTION_GET_CLASS (dist);
-	if (go_dist_klass->SUFFIX (get_ppf) != NULL)
-		return go_dist_klass->SUFFIX (get_ppf) (dist, x);
-	return SUFFIX (go_nan);
+	if (go_dist_klass->SUFFIX(get_ppf) != NULL)
+		return go_dist_klass->SUFFIX(get_ppf) (dist, x);
+
+	return SUFFIX(go_nan);
 }
 
 DOUBLE
-SUFFIX (go_distribution_get_hazard) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_distribution_get_hazard) (GODistribution *dist, DOUBLE x)
 {
-	GODistributionClass *go_dist_klass;
+	g_return_val_if_fail (GO_DISTRIBUTION (dist), SUFFIX(go_nan));
 
-	g_return_val_if_fail (GO_DISTRIBUTION (dist), SUFFIX (go_nan));
-
-	go_dist_klass = GO_DISTRIBUTION_GET_CLASS (dist);
-	if (go_dist_klass->SUFFIX (get_density) != NULL &&
-	    go_dist_klass->SUFFIX (get_cumulative) != NULL)
-		return go_dist_klass->SUFFIX (get_density) (dist, x) /
-			(1. - go_dist_klass->SUFFIX (get_cumulative) (dist, x));
-	return SUFFIX (go_nan);
+	return SUFFIX(go_distribution_get_density) (dist, x) /
+		SUFFIX(go_distribution_get_survival) (dist, x);
 }
 
 DOUBLE
-SUFFIX (go_distribution_get_cumulative_hazard) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_distribution_get_cumulative_hazard) (GODistribution *dist, DOUBLE x)
 {
-	GODistributionClass *go_dist_klass;
+	g_return_val_if_fail (GO_DISTRIBUTION (dist), SUFFIX(go_nan));
 
-	g_return_val_if_fail (GO_DISTRIBUTION (dist), SUFFIX (go_nan));
-
-	go_dist_klass = GO_DISTRIBUTION_GET_CLASS (dist);
-	if (go_dist_klass->SUFFIX (get_cumulative) != NULL)
-		return SUFFIX (log) (1. - go_dist_klass->SUFFIX (get_cumulative) (dist, x));
-	return SUFFIX (go_nan);
+	return -SUFFIX(log) (SUFFIX(go_distribution_get_survival) (dist, x));
 }
 
 DOUBLE
-SUFFIX (go_distribution_get_survival) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_distribution_get_survival) (GODistribution *dist, DOUBLE x)
 {
 	GODistributionClass *go_dist_klass;
 
-	g_return_val_if_fail (GO_DISTRIBUTION (dist), SUFFIX (go_nan));
+	g_return_val_if_fail (GO_DISTRIBUTION (dist), SUFFIX(go_nan));
 
 	go_dist_klass = GO_DISTRIBUTION_GET_CLASS (dist);
-	if (go_dist_klass->SUFFIX (get_cumulative) != NULL)
-		return 1. - go_dist_klass->SUFFIX (get_cumulative) (dist, x);
-	return SUFFIX (go_nan);
+	if (go_dist_klass->SUFFIX(get_cumulative) != NULL)
+		return go_dist_klass->SUFFIX(get_cumulative) (dist, FALSE, x);
+
+	return SUFFIX(go_nan);
 }
 
 DOUBLE
-SUFFIX (go_distribution_get_inverse_survival) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_distribution_get_inverse_survival) (GODistribution *dist, DOUBLE x)
 {
-	GODistributionClass *go_dist_klass;
+	g_return_val_if_fail (GO_DISTRIBUTION (dist), SUFFIX(go_nan));
+	g_return_val_if_fail (x >= 0 && x <= 1, SUFFIX(go_nan));
 
-	g_return_val_if_fail (GO_DISTRIBUTION (dist), SUFFIX (go_nan));
-
-	go_dist_klass = GO_DISTRIBUTION_GET_CLASS (dist);
-	if (go_dist_klass->SUFFIX (get_ppf) != NULL)
-		return 1. - go_dist_klass->SUFFIX (get_ppf) (dist, x);
-	return SUFFIX (go_nan);
+	return SUFFIX(go_distribution_get_ppf) (dist, 1 - x);
 }
 
 /*****************************************************************************/
 /*                        Normal distribution                              */
 /*****************************************************************************/
-#ifdef DISTRIBUTION_FIRST_PATH
+
+#ifdef DEFINE_COMMON
 
 typedef struct {
 	GODistribution base;
-
 } GODistNormal;
 
 #define GO_TYPE_DIST_NORMAL	  	(go_dist_normal_get_type ())
@@ -421,62 +446,54 @@ GType go_dist_normal_get_type (void);
 
 typedef GODistributionClass GODistNormalClass;
 
-#endif /* DISTRIBUTION_FIRST_PATH */
+#endif /* DEFINE_COMMON */
 
 static DOUBLE
-SUFFIX (go_normal_get_density) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_normal_get_density) (GODistribution *dist, DOUBLE x)
 {
-	return SUFFIX (go_dnorm) (x, dist->location, dist->scale, FALSE);
+	return SUFFIX(go_dnorm) (x, DIST_LOCATION, DIST_SCALE, FALSE);
 }
 
 static DOUBLE
-SUFFIX (go_normal_get_cumulative) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_normal_get_cumulative) (GODistribution *dist, gboolean lower_tail, DOUBLE x)
 {
-	return SUFFIX (go_pnorm) (x, dist->location, dist->scale, TRUE, FALSE);
+	return SUFFIX(go_pnorm) (x, DIST_LOCATION, DIST_SCALE, lower_tail, FALSE);
 }
 
 static DOUBLE
-SUFFIX (go_normal_get_ppf) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_normal_get_ppf) (GODistribution *dist, DOUBLE x)
 {
-	return SUFFIX (go_qnorm) (x, dist->location, dist->scale, TRUE, FALSE);
+	return SUFFIX(go_qnorm) (x, DIST_LOCATION, DIST_SCALE, TRUE, FALSE);
 }
 
-#ifdef DISTRIBUTION_LAST_PATH
-
-static void
-go_dist_normal_class_init (GObjectClass *klass)
-{
-	GODistributionClass *dist_klass = (GODistributionClass *) klass;
-	dist_klass->dist_type = GO_DISTRIBUTION_NORMAL;
-	dist_klass->get_density = go_normal_get_density;
-	dist_klass->get_cumulative = go_normal_get_cumulative;
-	dist_klass->get_ppf = go_normal_get_ppf;
-#ifdef GOFFICE_WITH_LONG_DOUBLE
-	dist_klass->get_densityl = go_normal_get_densityl;
-	dist_klass->get_cumulativel = go_normal_get_cumulativel;
-	dist_klass->get_ppfl = go_normal_get_ppfl;
-#endif
-}
+#if LAST_INCLUDE_PASS
 
 static void
 go_dist_normal_init (GODistribution *dist)
 {
 }
 
+static void
+go_dist_normal_class_init (GObjectClass *klass)
+{
+	GODistributionClass *dist_klass = (GODistributionClass *) klass;
+	dist_klass->dist_type = GO_DISTRIBUTION_NORMAL;
+	SET_HANDLERS(dist_klass, go_normal);
+}
+
 GSF_CLASS (GODistNormal, go_dist_normal,
 	   go_dist_normal_class_init, go_dist_normal_init,
 	   GO_TYPE_DISTRIBUTION)
 
-#endif /* DISTRIBUTION_LAST_PATH */
+#endif /* LAST_INCLUDE_PASS */
 
 /*****************************************************************************/
 /*                        Uniform distribution                              */
 /*****************************************************************************/
-#ifdef DISTRIBUTION_FIRST_PATH
+#ifdef DEFINE_COMMON
 
 typedef struct {
 	GODistribution base;
-
 } GODistUniform;
 
 #define GO_TYPE_DIST_UNIFORM	  	(go_dist_uniform_get_type ())
@@ -487,48 +504,43 @@ GType go_dist_uniform_get_type (void);
 
 typedef GODistributionClass GODistUniformClass;
 
-#endif /* DISTRIBUTION_FIRST_PATH */
+#endif /* DEFINE_COMMON */
 
 static DOUBLE
-SUFFIX (go_uniform_get_density) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_uniform_get_density) (GODistribution *dist, DOUBLE x)
 {
-	x = (x - dist->location) / dist->scale;
-	return (x >= 0. && x < 1.)? SUFFIX (1.) / dist->scale: SUFFIX (0.);
+	x = (x - DIST_LOCATION) / DIST_SCALE;
+	return (x >= 0 && x < 1)
+		? CONST (1.) / DIST_SCALE
+		: CONST (0.);
 }
 
 static DOUBLE
-SUFFIX (go_uniform_get_cumulative) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_uniform_get_cumulative) (GODistribution *dist, gboolean lower_tail, DOUBLE x)
 {
-	x = (x - dist->location) / dist->scale;
-	if (x < 0.)
-		return SUFFIX (0.);
-	else if (x >= 1.)
-		return SUFFIX (1.);
+	x = (x - DIST_LOCATION) / DIST_SCALE;
+	if (x < 0)
+		return lower_tail ? 0 : 1;
+	else if (x >= 1)
+		return lower_tail ? 1 : 0;
 	else
-		return x;
+		return lower_tail ? x : 1 - x;
 }
 
 static DOUBLE
-SUFFIX (go_uniform_get_ppf) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_uniform_get_ppf) (GODistribution *dist, DOUBLE x)
 {
-	return x * dist->scale + dist->location;
+	return x * DIST_SCALE + DIST_LOCATION;
 }
 
-#ifdef DISTRIBUTION_LAST_PATH
+#if LAST_INCLUDE_PASS
 
 static void
 go_dist_uniform_class_init (GObjectClass *klass)
 {
 	GODistributionClass *dist_klass = (GODistributionClass *) klass;
 	dist_klass->dist_type = GO_DISTRIBUTION_UNIFORM;
-	dist_klass->get_density = go_uniform_get_density;
-	dist_klass->get_cumulative = go_uniform_get_cumulative;
-	dist_klass->get_ppf = go_uniform_get_ppf;
-#ifdef GOFFICE_WITH_LONG_DOUBLE
-	dist_klass->get_densityl = go_uniform_get_densityl;
-	dist_klass->get_cumulativel = go_uniform_get_cumulativel;
-	dist_klass->get_ppfl = go_uniform_get_ppfl;
-#endif
+	SET_HANDLERS(dist_klass, go_uniform);
 }
 
 static void
@@ -540,16 +552,15 @@ GSF_CLASS (GODistUniform, go_dist_uniform,
 	   go_dist_uniform_class_init, go_dist_uniform_init,
 	   GO_TYPE_DISTRIBUTION)
 
-#endif /* DISTRIBUTION_LAST_PATH */
+#endif /* LAST_INCLUDE_PASS */
 
 /*****************************************************************************/
 /*                        Cauchy distribution                              */
 /*****************************************************************************/
-#ifdef DISTRIBUTION_FIRST_PATH
+#ifdef DEFINE_COMMON
 
 typedef struct {
 	GODistribution base;
-
 } GODistCauchy;
 
 #define GO_TYPE_DIST_CAUCHY	  	(go_dist_cauchy_get_type ())
@@ -560,41 +571,34 @@ GType go_dist_cauchy_get_type (void);
 
 typedef GODistributionClass GODistCauchyClass;
 
-#endif /* DISTRIBUTION_FIRST_PATH */
+#endif /* DEFINE_COMMON */
 
 static DOUBLE
-SUFFIX (go_cauchy_get_density) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_cauchy_get_density) (GODistribution *dist, DOUBLE x)
 {
-	return SUFFIX (go_dcauchy) (x, dist->location, dist->scale, FALSE);
+	return SUFFIX(go_dcauchy) (x, DIST_LOCATION, DIST_SCALE, FALSE);
 }
 
 static DOUBLE
-SUFFIX (go_cauchy_get_cumulative) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_cauchy_get_cumulative) (GODistribution *dist, gboolean lower_tail, DOUBLE x)
 {
-	return SUFFIX (go_pcauchy) (x, dist->location, dist->scale, TRUE, FALSE);
+	return SUFFIX(go_pcauchy) (x, DIST_LOCATION, DIST_SCALE, lower_tail, FALSE);
 }
 
 static DOUBLE
-SUFFIX (go_cauchy_get_ppf) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_cauchy_get_ppf) (GODistribution *dist, DOUBLE x)
 {
-	return SUFFIX (go_qcauchy) (x, dist->location, dist->scale, TRUE, FALSE);
+	return SUFFIX(go_qcauchy) (x, DIST_LOCATION, DIST_SCALE, TRUE, FALSE);
 }
 
-#ifdef DISTRIBUTION_LAST_PATH
+#if LAST_INCLUDE_PASS
 
 static void
 go_dist_cauchy_class_init (GObjectClass *klass)
 {
 	GODistributionClass *dist_klass = (GODistributionClass *) klass;
 	dist_klass->dist_type = GO_DISTRIBUTION_CAUCHY;
-	dist_klass->get_density = go_cauchy_get_density;
-	dist_klass->get_cumulative = go_cauchy_get_cumulative;
-	dist_klass->get_ppf = go_cauchy_get_ppf;
-#ifdef GOFFICE_WITH_LONG_DOUBLE
-	dist_klass->get_densityl = go_cauchy_get_densityl;
-	dist_klass->get_cumulativel = go_cauchy_get_cumulativel;
-	dist_klass->get_ppfl = go_cauchy_get_ppfl;
-#endif
+	SET_HANDLERS(dist_klass, go_cauchy);
 }
 
 static void
@@ -606,12 +610,12 @@ GSF_CLASS (GODistCauchy, go_dist_cauchy,
 	   go_dist_cauchy_class_init, go_dist_cauchy_init,
 	   GO_TYPE_DISTRIBUTION)
 
-#endif /* DISTRIBUTION_LAST_PATH */
+#endif /* LAST_INCLUDE_PASS */
 
 /*****************************************************************************/
 /*                        Weibull distribution                              */
 /*****************************************************************************/
-#ifdef DISTRIBUTION_FIRST_PATH
+#ifdef DEFINE_COMMON
 
 typedef struct {
 	GODistribution base;
@@ -631,27 +635,28 @@ enum {
 	WEIBULL_PROP_SHAPE
 };
 
-#endif /* DISTRIBUTION_FIRST_PATH */
+#endif /* DEFINE_COMMON */
 
 static DOUBLE
-SUFFIX (go_weibull_get_density) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_weibull_get_density) (GODistribution *dist, DOUBLE x)
 {
-	return SUFFIX (go_dweibull) (x - dist->location, GO_DIST_WEIBULL (dist)->shape, dist->scale, FALSE) ;
+	return SUFFIX(go_dweibull) (x - DIST_LOCATION, GO_DIST_WEIBULL (dist)->shape, DIST_SCALE, FALSE) ;
 }
 
 static DOUBLE
-SUFFIX (go_weibull_get_cumulative) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_weibull_get_cumulative) (GODistribution *dist, gboolean lower_tail, DOUBLE x)
 {
-	return SUFFIX (go_pweibull) (x - dist->location, GO_DIST_WEIBULL (dist)->shape, dist->scale, TRUE, FALSE) ;
+	return SUFFIX(go_pweibull) (x - DIST_LOCATION, GO_DIST_WEIBULL (dist)->shape, DIST_SCALE,
+				    lower_tail, FALSE) ;
 }
 
 static DOUBLE
-SUFFIX (go_weibull_get_ppf) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_weibull_get_ppf) (GODistribution *dist, DOUBLE x)
 {
-	return SUFFIX (go_qweibull) (x, GO_DIST_WEIBULL (dist)->shape, dist->scale, TRUE, FALSE) + dist->location;
+	return SUFFIX(go_qweibull) (x, GO_DIST_WEIBULL (dist)->shape, DIST_SCALE, TRUE, FALSE) + DIST_LOCATION;
 }
 
-#ifdef DISTRIBUTION_LAST_PATH
+#if LAST_INCLUDE_PASS
 
 static void
 go_dist_weibull_set_property (GObject *obj, guint param_id,
@@ -688,14 +693,7 @@ go_dist_weibull_class_init (GObjectClass *klass)
 {
 	GODistributionClass *dist_klass = (GODistributionClass *) klass;
 	dist_klass->dist_type = GO_DISTRIBUTION_WEIBULL;
-	dist_klass->get_density = go_weibull_get_density;
-	dist_klass->get_cumulative = go_weibull_get_cumulative;
-	dist_klass->get_ppf = go_weibull_get_ppf;
-#ifdef GOFFICE_WITH_LONG_DOUBLE
-	dist_klass->get_densityl = go_weibull_get_densityl;
-	dist_klass->get_cumulativel = go_weibull_get_cumulativel;
-	dist_klass->get_ppfl = go_weibull_get_ppfl;
-#endif
+	SET_HANDLERS(dist_klass, go_weibull);
 	klass->set_property = go_dist_weibull_set_property;
 	klass->get_property = go_dist_weibull_get_property;
 	g_object_class_install_property (klass, WEIBULL_PROP_SHAPE,
@@ -718,12 +716,12 @@ GSF_CLASS (GODistWeibull, go_dist_weibull,
 	   go_dist_weibull_class_init, go_dist_weibull_init,
 	   GO_TYPE_DISTRIBUTION)
 
-#endif /* DISTRIBUTION_LAST_PATH */
+#endif /* LAST_INCLUDE_PASS */
 
 /*****************************************************************************/
 /*                        Lognormal distribution                              */
 /*****************************************************************************/
-#ifdef DISTRIBUTION_FIRST_PATH
+#ifdef DEFINE_COMMON
 
 typedef struct {
 	GODistribution base;
@@ -743,31 +741,32 @@ enum {
 	LNORM_PROP_SHAPE
 };
 
-#endif /* DISTRIBUTION_FIRST_PATH */
+#endif /* DEFINE_COMMON */
 
 static DOUBLE
-SUFFIX (go_log_normal_get_density) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_log_normal_get_density) (GODistribution *dist, DOUBLE x)
 {
-	return SUFFIX (go_dlnorm) ((x - dist->location) / dist->scale, 0., GO_DIST_LOG_NORMAL (dist)->shape, FALSE);
+	return SUFFIX(go_dlnorm) ((x - DIST_LOCATION) / DIST_SCALE, 0., GO_DIST_LOG_NORMAL (dist)->shape, FALSE);
 }
 
 static DOUBLE
-SUFFIX (go_log_normal_get_cumulative) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_log_normal_get_cumulative) (GODistribution *dist, gboolean lower_tail, DOUBLE x)
 {
-	return SUFFIX (go_plnorm) ((x - dist->location) / dist->scale, 0., GO_DIST_LOG_NORMAL (dist)->shape, TRUE, FALSE);
+	return SUFFIX(go_plnorm) ((x - DIST_LOCATION) / DIST_SCALE, 0., GO_DIST_LOG_NORMAL (dist)->shape,
+				  lower_tail, FALSE);
 }
 
 static DOUBLE
-SUFFIX (go_log_normal_get_ppf) (GODistribution *dist, DOUBLE x)
+SUFFIX(go_log_normal_get_ppf) (GODistribution *dist, DOUBLE x)
 {
-	return SUFFIX (go_qlnorm) (x, 0., GO_DIST_LOG_NORMAL (dist)->shape, TRUE, FALSE) * dist->scale + dist->location;
+	return SUFFIX(go_qlnorm) (x, 0., GO_DIST_LOG_NORMAL (dist)->shape, TRUE, FALSE) * DIST_SCALE + DIST_LOCATION;
 }
 
-#ifdef DISTRIBUTION_LAST_PATH
+#if LAST_INCLUDE_PASS
 
 static void
 go_dist_log_normal_set_property (GObject *obj, guint param_id,
-			      GValue const *value, GParamSpec *pspec)
+				 GValue const *value, GParamSpec *pspec)
 {
 	GODistLogNormal *dist = GO_DIST_LOG_NORMAL (obj);
 
@@ -800,14 +799,7 @@ go_dist_log_normal_class_init (GObjectClass *klass)
 {
 	GODistributionClass *dist_klass = (GODistributionClass *) klass;
 	dist_klass->dist_type = GO_DISTRIBUTION_LOGNORMAL;
-	dist_klass->get_density = go_log_normal_get_density;
-	dist_klass->get_cumulative = go_log_normal_get_cumulative;
-	dist_klass->get_ppf = go_log_normal_get_ppf;
-#ifdef GOFFICE_WITH_LONG_DOUBLE
-	dist_klass->get_densityl = go_log_normal_get_densityl;
-	dist_klass->get_cumulativel = go_log_normal_get_cumulativel;
-	dist_klass->get_ppfl = go_log_normal_get_ppfl;
-#endif
+	SET_HANDLERS(dist_klass, go_log_normal);
 	klass->set_property = go_dist_log_normal_set_property;
 	klass->get_property = go_dist_log_normal_get_property;
 	g_object_class_install_property (klass, LNORM_PROP_SHAPE,
@@ -830,13 +822,13 @@ GSF_CLASS (GODistLogNormal, go_dist_log_normal,
 	   go_dist_log_normal_class_init, go_dist_log_normal_init,
 	   GO_TYPE_DISTRIBUTION)
 
-#endif /* DISTRIBUTION_LAST_PATH */
+#endif /* LAST_INCLUDE_PASS */
 
 /*****************************************************************************/
 /*                        Global functions                                   */
 /*****************************************************************************/
 
-#ifdef DISTRIBUTION_LAST_PATH
+#if LAST_INCLUDE_PASS
 
 GODistribution*
 go_distribution_new (GODistributionType type)
@@ -879,4 +871,12 @@ _go_distributions_init (void)
 	go_dist_log_normal_get_type ();
 }
 
-#endif /* DISTRIBUTION_LAST_PATH */
+#endif /* LAST_INCLUDE_PASS */
+
+/* ------------------------------------------------------------------------- */
+
+// See comments at top
+#endif // SKIP_THIS_PASS
+#if INCLUDE_PASS < INCLUDE_PASS_LAST
+#include __FILE__
+#endif
