@@ -1,4 +1,3 @@
-/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * go-svg.h - SVG image support
  *
@@ -22,6 +21,7 @@
 
 #ifndef GO_SVG_H
 #define GO_SVG_H
+#ifdef GOFFICE_WITH_LIBRSVG
 
 #include <goffice/goffice.h>
 
@@ -38,4 +38,5 @@ GOImage *go_svg_new_from_data (char const *data, size_t length, GError **error);
 
 G_END_DECLS
 
+#endif /* GOFFICE_WITH_LIBRSVG */
 #endif /* GO_SVG_H */
